@@ -90,7 +90,8 @@ class OpenFile(object):
         return "<OpenFile '{}'>".format(self.path)
 
     def __fspath__(self):
-        return self.path
+        # may raise if cannot be resolved to local file
+        return self.open().__fspath__()
 
     def __enter__(self):
         mode = self.mode.replace("t", "").replace("b", "") + "b"
@@ -196,6 +197,26 @@ def open_files(
     -------
     List of ``OpenFile`` objects.
     """
+    chain = _un_chain(urlpath, kwargs)
+    if len(chain) > 1:
+        kwargs = chain[0][2]
+        inkwargs = kwargs
+        urlpath = False
+        for i, ch in enumerate(chain):
+            urls, protocol, kw = ch
+            if isinstance(urls, str):
+                if not urlpath and split_protocol(urls)[1]:
+                    urlpath = protocol + "://" + split_protocol(urls)[1]
+            else:
+                if not urlpath and any(split_protocol(u)[1] for u in urls):
+                    urlpath = [protocol + "://" + split_protocol(u)[1] for u in urls]
+            if i == 0:
+                continue
+            inkwargs["target_protocol"] = protocol
+            inkwargs["target_options"] = kw.copy()
+            inkwargs["fo"] = urls
+            inkwargs = inkwargs["target_options"]
+        protocol = chain[0][1]
     fs, fs_token, paths = get_fs_token_paths(
         urlpath,
         mode,
@@ -218,6 +239,32 @@ def open_files(
     ]
 
 
+def _un_chain(path, kwargs):
+    if isinstance(path, (tuple, list)):
+        bits = [_un_chain(p, kwargs) for p in path]
+        out = []
+        for pbit in zip(*bits):
+            paths, protocols, kwargs = zip(*pbit)
+            if len(set(protocols)) > 1:
+                raise ValueError("Protocol mismatch in URL chain")
+            if len(set(paths)) == 1:
+                paths = paths[0]
+            else:
+                paths = list(paths)
+            out.append([paths, protocols[0], kwargs[0]])
+        return out
+    bits = [p if "://" in p else p + "://" for p in path.split("::")]
+    # [[url, protocol, kwargs], ...]
+    return [
+        (
+            bit,
+            split_protocol(bit)[0] or "file",
+            kwargs.get(split_protocol(bit)[0] or "file", {}),
+        )
+        for bit in bits
+    ]
+
+
 def open(
     urlpath,
     mode="rb",
@@ -278,6 +325,29 @@ def open(
     )[0]
 
 
+def open_local(url, mode="rb", **storage_options):
+    """Open file(s) which can be resolved to local
+
+    For files which either are local, or get downloaded upon open
+    (e.g., by file caching)
+
+    Parameters
+    ----------
+    url: str or list(str)
+    mode: str
+        Must be read mode
+    storage_options:
+        passed on to FS for or used by open_files (e.g., compression)
+    """
+    if "r" not in mode:
+        raise ValueError("Can only ensure local files when reading")
+    of = open_files(url, mode=mode, **storage_options)
+    paths = [f.open().name for f in of]
+    if isinstance(url, str) and "*" not in url:
+        return paths[0]
+    return paths
+
+
 def get_compression(urlpath, compression):
     if compression == "infer":
         compression = infer_compression(urlpath)
@@ -368,12 +438,13 @@ def get_fs_token_paths(
         if not urlpath:
             raise ValueError("empty urlpath sequence")
         protocols, paths = zip(*map(split_protocol, urlpath))
-        protocol = protocol or protocols[0]
-        if not all(p == protocol for p in protocols):
-            raise ValueError(
-                "When specifying a list of paths, all paths must "
-                "share the same protocol"
-            )
+        if protocol is None:
+            protocol = protocols[0]
+            if not all(p == protocol for p in protocols):
+                raise ValueError(
+                    "When specifying a list of paths, all paths must "
+                    "share the same protocol"
+                )
         cls = get_filesystem_class(protocol)
         optionss = list(map(cls._get_kwargs_from_urls, urlpath))
         paths = [cls._strip_protocol(u) for u in urlpath]
